@@ -14,6 +14,8 @@ our @EXPORT      = qw(REGION_INSIDE REGION_OUTSIDE
 		      select_regions
 		      merge_regions
 		      reverse_regions
+		      match_borders
+		      borders_to_regions
 		      );
 our %EXPORT_TAGS = ( );
 our @EXPORT_OK   = qw();
@@ -42,6 +44,14 @@ sub new {
     $obj;
 }
 
+sub configure {
+    my $obj = shift;
+    while (@_ >= 2) {
+	$obj->{$_[0]} = $_[1];
+	splice @_, 0, 2;
+    }
+}
+
 sub spec         { shift -> {SPEC} }
 sub flag         { shift -> {FLAG} }
 sub is_union     { (shift->flag & REGION_SET_MASK ) == REGION_UNION     }
@@ -75,14 +85,6 @@ package App::Greple::Regions::Holder {
     }
 }
 
-sub configure {
-    my $obj = shift;
-    while (@_ >= 2) {
-	$obj->{$_[0]} = $_[1];
-	splice @_, 0, 2;
-    }
-}
-
 sub match_regions {
     my %arg = @_;
     my $pattern = $arg{pattern} // croak "Parameter error";
@@ -250,4 +252,22 @@ sub reverse_regions {
     grep { $_->[0] != $_->[1] } @reverse
 }
 
+sub match_borders {
+    my $regex = shift;
+    my @border = (0);
+    while (/$regex/gp) {
+	my $pos = pos();
+	for my $i ($pos - length ${^MATCH}, $pos) {
+	    push @border, $i if $border[-1] != $i;
+	}
+    }
+    push @border, length if $border[-1] != length;
+    @border;
+}
+
+sub borders_to_regions {
+    return () if @_ < 2;
+    map { [ $_[$_-1], $_[$_] ] } 1..$#_;
+}
+
 1;
